/*
 *  main.cpp
 *
 *  Copyright (C) 2004 Jonathan Riddell <jr@jriddell.org>
 *
 *  Based on kcm_kvaio
 *  Copyright (C) 2003 Mirko Boehm (mirko@kde.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#include <QLabel>
#include <QLayout>
#include <QCheckBox>
#include <QSpinBox>
#include <QFile>
#include <q3buttongroup.h>
//Added by qt3to4:
#include <QVBoxLayout>
#include <QDBusInterface>
#include <QDBusReply>

#include <kaboutdata.h>
#include <kdebug.h>
#include <kconfig.h>
#include <kurlrequester.h>
#include <kcmodule.h>
#include <kstandarddirs.h>
#include <kpluginfactory.h>
#include <kpluginloader.h>


#ifdef Q_OS_FREEBSD
#include <sys/types.h>
#include <sys/sysctl.h>
#endif

#include "kcmthinkpad_general.h"

#include "main.h"
#include "main.moc"

K_PLUGIN_FACTORY(KCMThinkpadModuleFactory, registerPlugin<KCMThinkpadModule>();)
K_EXPORT_PLUGIN(KCMThinkpadModuleFactory("kcmthinkpad"))


#define CONFIG_FILE "kmilodrc"

KCMThinkpadModule::KCMThinkpadModule(QWidget* parent, const QVariantList&)
	: KCModule(KCMThinkpadModuleFactory::componentData(), parent) {
	KAboutData* about =
		new KAboutData(I18N_NOOP("kcmthinkpad"), 0,
			       ki18n("KDE Control Module for IBM Thinkpad "
					 "Laptop Hardware"),
			       0, KLocalizedString(), KAboutData::License_GPL,
			       ki18n("(c) 2004 Jonathan Riddell"));

	about->addAuthor(ki18n("Jonathan Riddell"),
			 ki18n("Original author"),
			 "jr@jriddell.org");
	setAboutData( about );

	QVBoxLayout* layout = new QVBoxLayout(this);
	m_KCMThinkpadGeneral = new KCMThinkpadGeneral(this);
	layout->addWidget( m_KCMThinkpadGeneral );
	layout->addStretch();

	load();

	//try and open /dev/nvram
	m_nvramReadable = false;
	m_nvramWriteable = false;

#ifdef Q_OS_FREEBSD
	// Look if the sysctl tree of acpi_ibm is in place
	u_int n = 0;
	size_t len = sizeof(n);
	if (m_nvramReadable = ( sysctlbyname("dev.acpi_ibm.0.volume", &n, &len, NULL, 0) != -1 ))
		m_nvramWriteable = ( sysctlbyname("dev.acpi_ibm.0.volume", NULL, NULL, &n, len) != -1 );

	if (!m_nvramReadable) {
		setButtons(buttons() & ~Default);
		m_KCMThinkpadGeneral->bgGeneral->setEnabled(false);
		m_KCMThinkpadGeneral->tlOff->setText(i18n("In order to use the Thinkpad Buttons KMilo Plugin, "
							  "you have to load the acpi_ibm(4) driver."));
	} else if (!m_nvramWriteable) {
		m_KCMThinkpadGeneral->tlOff->setText(i18n("Could not write to dev.acpi_ibm.0.volume. "
							  "Using software volume, required for "
							  "R30/R31 models, or using a custom volume "
							  "change step is disabled."));
#else
	QFile nvramFile(m_nvramFile);
	if ( nvramFile.open(QIODevice::ReadOnly) )  {
		m_nvramReadable = true;
		nvramFile.close();
	}
	if ( nvramFile.open(QIODevice::WriteOnly) )  {
		m_nvramWriteable = true;
		nvramFile.close();
	}

	if (!m_nvramReadable) {
            setButtons(buttons() & ~Default);
            m_KCMThinkpadGeneral->bgGeneral->setEnabled(false);
	} else if (!m_nvramWriteable) {
		m_KCMThinkpadGeneral->tlOff->setText(i18n("Could not write to %1. "
							  "To use the software volume, required for "
							  "R30/R31 models and to use a custom volume "
							  "change step, set the nvram device to world "
							  "writeable: <em>chmod 666 "
							  "/dev/nvram</em>", m_nvramFile));
#endif
	} else {
		m_KCMThinkpadGeneral->tlOff->setText(i18n("Thinkpad Buttons KMilo Plugin Ready For Configuration"));
	}

	connect( m_KCMThinkpadGeneral, SIGNAL(changed()), SLOT(changed()));

}

void KCMThinkpadModule::save() {
    if (!m_nvramReadable)
        return;

    KConfig kconfig(CONFIG_FILE);
    KConfigGroup config(&kconfig, "thinkpad");

    config.writeEntry("run", m_KCMThinkpadGeneral->mCbRun->isChecked());
    config.writeEntry("softwareVolume", m_KCMThinkpadGeneral->mCbSoftwareVolume->isChecked());
    config.writeEntry("volumeStep", m_KCMThinkpadGeneral->mSpinboxVolumeStep->value());
    config.writeEntry("buttonThinkpad", m_KCMThinkpadGeneral->commandExec->url().path());
    config.writeEntry("buttonHome", m_KCMThinkpadGeneral->commandExecHome->url().path());
    config.writeEntry("buttonSearch", m_KCMThinkpadGeneral->commandExecSearch->url().path());
    config.writeEntry("buttonMail", m_KCMThinkpadGeneral->commandExecMail->url().path());
    config.writeEntry("buttonZoom", m_KCMThinkpadGeneral->commandExecZoom->url().path());
    config.sync();

    QDBusInterface kmilod("org.kde.kded", "/modules/kmilod", "org.kde.kmilod");
    QDBusReply<void> reply = kmilod.call("reconfigure");
    if( !reply.isValid() )
         kDebug() << "KCMThinkpad::showTextMsg: "
                  << "there was an error using D-Bus on kmilod::reconfigure()." << endl;
}

void KCMThinkpadModule::load() {
    KConfig kconfig(CONFIG_FILE);
    KConfigGroup config(&kconfig, "thinkpad");

    m_KCMThinkpadGeneral->mCbRun->setChecked(config.readEntry("run", false));
    m_KCMThinkpadGeneral->mCbSoftwareVolume->setChecked(config.readEntry("softwareVolume", true));
    m_KCMThinkpadGeneral->mSpinboxVolumeStep->setValue(config.readEntry("volumeStep", 14));
    m_KCMThinkpadGeneral->commandExec->setUrl(config.readEntry("buttonThinkpad",
                KStandardDirs::locate("exe", "konsole)")));
    m_KCMThinkpadGeneral->commandExecHome->setUrl(config.readEntry("buttonHome",
                KStandardDirs::locate("exe", "konqueror")));
    m_KCMThinkpadGeneral->commandExecSearch->setUrl(config.readEntry("buttonSearch",
                KStandardDirs::locate("exe", "kfind")));
    m_KCMThinkpadGeneral->commandExecMail->setUrl(config.readEntry("buttonMail",
                KStandardDirs::locate("exe", "kmail")));
    m_KCMThinkpadGeneral->commandExecZoom->setUrl(config.readEntry("buttonZoom", 
                KStandardDirs::locate("exe", "ksnapshot")));
    m_nvramFile = config.readEntry("nvram", "/dev/nvram");
}

void KCMThinkpadModule::defaults() {
    m_KCMThinkpadGeneral->mCbRun->setChecked(false);
    m_KCMThinkpadGeneral->mCbSoftwareVolume->setChecked(true);
    m_KCMThinkpadGeneral->mSpinboxVolumeStep->setValue(14);
    m_KCMThinkpadGeneral->commandExec->setUrl(KUrl::fromPath(KStandardDirs::locate("exe", "konsole")));
    m_KCMThinkpadGeneral->commandExecHome->setUrl(KUrl::fromPath(KStandardDirs::locate("exe",
                    "konqueror")));
    m_KCMThinkpadGeneral->commandExecSearch->setUrl(KUrl::fromPath(KStandardDirs::locate("exe",
                    "kfind")));
    m_KCMThinkpadGeneral->commandExecMail->setUrl(KUrl::fromPath(KStandardDirs::locate("exe",
                    "kmail")));
    m_KCMThinkpadGeneral->commandExecZoom->setUrl(KUrl::fromPath(KStandardDirs::locate("exe",
                "ksnapshot")));
}

